@@ -16,10 +16,14 @@
 # 02110-1301, USA.
 
 from math import pi
+import pygame
 from trosnoth.src.utils.utils import timeNow
 
 class ImageCollection(object):
     def __init__(self, *images):
+        for image in images:
+            if not isinstance(image, pygame.Surface):
+                raise TypeError('%r is not a pygame.Surface' % (image,))
         self.images = images
 
     def getImage(self):
@@ -90,7 +94,7 @@ class AnimationWithStart(Animation):
         if elapsed > self.speed * len(self.images):
             self.start -= self.speed * len(self.images)
             self.initial = False
-        if initial:
+        if self.initial:
             return self.initialAnimation.getImage()
         else:
             return self.images[imgIndex]
