@@ -223,8 +223,8 @@ class HoverButton(Element):
             self.mouseOver = rect.collidepoint(event.pos)
 
         # Active events.
-        if event.type == pgl.MOUSEBUTTONDOWN and \
-                rect.collidepoint(event.pos):
+        if event.type == pgl.MOUSEBUTTONDOWN and event.button == 1 and \
+           rect.collidepoint(event.pos):
             self.onClick.execute(self)
         elif event.type == pgl.KEYDOWN and event.key == self.hotkey:
             self.onClick.execute(self)
