@@ -70,9 +70,9 @@ class TabHeader(Element):
         self._redraw()
 
     def processEvent(self, event):
-        if (not self.hasFocus and \
-           event.type == pygame.locals.MOUSEBUTTONDOWN and \
-           pygame.Rect(self.pos, self.size).collidepoint(event.pos)): 
+        if (not self.hasFocus and event.type == pygame.locals.MOUSEBUTTONDOWN \
+            and event.button == 1 and \
+            pygame.Rect(self.pos, self.size).collidepoint(event.pos)): 
             self.tabContainer._tabSelected(self.tab)
         else:
             return event
