@@ -23,3 +23,5 @@ class FireShot(Message):
 class ShotFired(Message):
     fields = 'player', 'shot'
 
+class ShotGone(Message):
+    fields = 'shot',    
\ No newline at end of file
