@@ -17,9 +17,6 @@
 
 from trosnoth.src.utils.components import Message
 
-class StartingTransaction(Message):
-    fields = 'team', 'player', 'numStars', 'upgradeType'
-
 class AddingStars(Message):
     fields = 'teamd', 'player', 'numStars'
 
@@ -27,19 +24,32 @@ class StartedTransaction(Message):
     fields = 'transaction',
 
 class AddedStars(Message):
-    fields = 'team', 'player', 'numStars', 'totalStars'
+    fields = 'team', 'player', 'numStars'
 
 class TransactionComplete(Message):
-    fields = 'team', 'player', 'upgradeType'
+    fields = 'team', 'player', 'upgradeType', 'transaction'
 
 class DeleteUpgrade(Message):
-    fields = 'player',
-    
-class AbandonTransaction(Message):
-    fields = 'team',
+    fields = 'player', 'upgrade'
 
 class UseUpgrade(Message):
     fields = 'player',
+    
+class AbandonTransaction(Message):
+    fields = 'transaction', 'reason'
+
 
 class TransactionChanged(Message):
     fields = 'transaction',
+
+class MinimapStarted(Message):
+    fields = 'player',
+
+class MinimapEnded(Message):
+    fields = 'player'
+
+class TurretStarted(Message):
+    fields = 'player',
+
+class TurretEnded(Message):
+    fields = 'player',
\ No newline at end of file
