@@ -17,10 +17,13 @@
 
 import pygame
 from trosnoth.src.model.obstacles import *
-from trosnoth.src.trosnothgui.ingame.colours import *
 from trosnoth.src.model.map import MapLayout
 from trosnoth.src.model.universe_base import GameState
 
+from trosnoth.src.messages.gameplay import PlayerKilled, KillShot
+# TODO: remove this blight upon Trosnoth:
+from trosnoth.src.network.gameplay import ZoneChangeMsg
+
 # TODO: Don't use isinstance so we don't have to use this
 from trosnoth.src.model.shot import Shot, GrenadeShot
 from trosnoth.src.model.player import Player
@@ -191,12 +194,10 @@ class MapBlock(object):
                 if self.collideTrajectory(shot.pos, player.pos,
                                           (deltaX, deltaY), 20):
                     if player.phaseshift:
-                        self.universe.onShotDestroyed.execute(shot)
+                        self.universe.requestPlug.send(KillShot(shot.originatingPlayer, shot))
                     else:
-                        self.universe.onPlayerKilled.execute(player, \
-                                                    shot.originatingPlayer,
-                                                    shot)
-
+                        self.universe.requestPlug.send(PlayerKilled(player, shot.originatingPlayer, shot))
+                        
     def checkPlayerCollisions(self, shot, deltaX, deltaY):
         '''Check for collisions of the specified shot with living local
         players. DeltaX and deltaY specify the path along which the shot is
@@ -207,10 +208,7 @@ class MapBlock(object):
                     # FIXME: It would be better not to have a hard-coded 20.
                     if self.collideTrajectory(player.pos, shot.pos,
                                               (deltaX, deltaY), 20):
-                        print 'player killed 2'
-                        self.universe.onPlayerKilled.execute(player, \
-                                                            shot.originatingPlayer, \
-                                                            shot)
+                        self.universe.requestPlug.send(PlayerKilled(player, shot.originatingPlayer, shot))
 
     def collideTrajectory(self, pt, origin, trajectory, tolerance):
         '''Returns true if pt lies within a distance of tolerance from the
@@ -263,9 +261,15 @@ class MapBlock(object):
         if unit.solid():
             # Check for collisions with obstacles - find the closest obstacle.
             for obstacle in self.getObstacles(type(unit)):
-                if isinstance(unit, Player) and unit._ignore == obstacle:
-                    continue
-                dX, dY = obstacle.collide(unit.pos, deltaX, deltaY)
+                soft = False
+                if isinstance(unit, Player):
+                    if unit._ignore == obstacle:
+                        continue
+                    elif (isinstance(unit._onGround, GroundObstacle) and
+                            (obstacle == unit._onGround.leftGround or obstacle
+                            == unit._onGround.rightGround)):
+                        soft = True
+                dX, dY = obstacle.collide(unit.pos, deltaX, deltaY, soft=soft)
                 if (dX, dY) != (deltaX, deltaY):
                     # Remember the last obstacle we hit.
                     lastObstacle = obstacle
@@ -348,7 +352,7 @@ class MapBlock(object):
         if newZone:
             # Zone needs to know which players are in it.
             unit.changeZone(newZone)
-            self.universe.onZoneChanged.execute(unit, newZone)
+            self.universe.requestPlug.send(ZoneChangeMsg(unit.id, newZone.id))
 
         return lastObstacle
             
@@ -476,6 +480,61 @@ class MapBlock(object):
             self._obstacleEdge(nextPt, curCnr)
             curCnr = (curCnr + 1) % 8
         
+    def coalesceLayout(self):
+        '''
+        Called once after all obstacles are added to the block. Used to
+        connect ground obstacles so the player can walk between them.
+        '''
+        leftCorners = {}    # (x, y) -> obstacle
+        rightCorners = {}    # (x, y) -> obstacle
+        toRemove = set()
+        for obstacle in self.plObstacles:
+            if not isinstance(obstacle, GroundObstacle):
+                continue
+            left = obstacle.pt1
+            right = obstacle.pt2
+            if left in leftCorners:
+                other = leftCorners[left]
+                if obstacle.ratio[1] < other.ratio[1]:
+                    leftCorners[left] = obstacle
+                elif obstacle.ratio[1] == other.ratio[1]:
+                    if obstacle.deltaPt[0] >= other.deltaPt[0]:
+                        toRemove.add(other)
+                        leftCorners[left] = obstacle
+                        if rightCorners[other.pt2] == other:
+                            del rightCorners[other.pt2]
+                    else:
+                        toRemove.add(obstacle)
+                        continue
+            else:
+                leftCorners[left] = obstacle
+            if right in rightCorners:
+                other = rightCorners[right]
+                if obstacle.ratio[1] > other.ratio[1]:
+                    rightCorners[right] = obstacle
+                elif obstacle.ratio[1] == other.ratio[1]:
+                    if obstacle.deltaPt[0] >= other.deltaPt[0]:
+                        toRemove.add(other)
+                        rightCorners[right] = obstacle
+                        if leftCorners[other.pt1] == other:
+                            del leftCorners[other.pt1]
+                    else:
+                        toRemove.add(obstacle)
+                        if leftCorners[obstacle.pt1] == obstacle:
+                            del leftCorners[obstacle.pt1]
+                        continue
+            else:
+                rightCorners[right] = obstacle
+
+        for obstacle in list(self.plObstacles):
+            if obstacle in toRemove:
+                self.plObstacles.remove(obstacle)
+                continue
+            if not isinstance(obstacle, GroundObstacle):
+                continue
+            obstacle.leftGround = rightCorners.get(obstacle.pt1)
+            obstacle.rightGround = leftCorners.get(obstacle.pt2)
+
     def _obstacle(self, pt0, pt1, cnr, kind=None):
         '''(kind, pt, delta) - internal. Adds a barrier for shots and players.
         kind        the class of obstacle to add for players.
@@ -551,7 +610,7 @@ class BodyMapBlock(MapBlock):
     def draw(self, surface, area):
         # First draw the background the colour of the zone.
         if self.zone:
-            clr = self.zone.getBackgroundColour()
+            clr = getZoneBackgroundColour(self.zone)
             surface.fill(clr, area)
 
         # Now draw the obstacles.
@@ -563,6 +622,14 @@ class BodyMapBlock(MapBlock):
             r.center = self.orbPos()
             surface.blit(img, r)
 
+# TODO: Separate the drawing from the model.
+def getZoneBackgroundColour(zone):
+    colours = zone.app.theme.colours
+    if zone.zoneOwner:
+        return colours.backgroundColour(zone.zoneOwner)
+    else:
+        return colours.neutral_bg
+
 
 class TopBodyMapBlock(BodyMapBlock):
     '''Represents a map block containing the top half of a zone.'''
@@ -596,6 +663,8 @@ class InterfaceMapBlock(MapBlock):
 
     def __init__(self, universe, defn, zone1, zone2):
         super(InterfaceMapBlock, self).__init__(universe, defn)
+        self.app = universe.app
+        self._makeInterfaceSurfaces()
 
         self.zone1 = zone1
         self.zone2 = zone2
@@ -617,7 +686,7 @@ class InterfaceMapBlock(MapBlock):
         if self.zone1 and self.zone2 and \
            self.zone1.zoneOwner == self.zone2.zoneOwner:
             # Same colour; simply fill in that colour
-            clr = self.zone1.getBackgroundColour()
+            clr = getZoneBackgroundColour(self.zone1)
             surface.fill(clr, area)
         else:
             if not self.zone1:
@@ -636,7 +705,7 @@ class InterfaceMapBlock(MapBlock):
                     z2 = '\x00'
                     
             # This is getting a pre-made image for the background colour
-            img = interfaceSurfaces[type(self)][(z1, z2)]
+            img = self.app.__interfaceSurfaces[type(self)][(z1, z2)]
 
             rp = (area.left - self.drawRect.left, area.top - self.drawRect.top)
             rr = pygame.rect.Rect(rp, area.size)
@@ -644,6 +713,57 @@ class InterfaceMapBlock(MapBlock):
 
         # Now draw the obstacles.
         super(InterfaceMapBlock, self).draw(surface, area)
+
+    def _makeInterfaceSurfaces(self):
+        if hasattr(self.app, '__interfaceSurfaces'):
+            return
+        c = self.app.theme.colours
+        options = ('A', 'B', '\x00', None)
+        colours = {'A' : c.team1bg,
+                   'B' : c.team2bg,
+                   '\x00' : c.neutral_bg,
+                   None : (0,0,0)}
+        forw = {}
+        backw = {}
+        i = 0
+        while i < len(options):
+            # Skip ('A', 'B') by always starting j at least at '\x00'
+            j = max(i + 1, 2)
+            clr1 = colours[options[i]]
+            while j < len(options):
+                clr2 = colours[options[j]]
+                
+                img = pygame.surface.Surface((MapLayout.zoneInterfaceWidth, \
+                                              MapLayout.halfZoneHeight))
+                rect = img.get_rect()
+                pts1 = (rect.topleft, rect.bottomright, rect.bottomleft)
+                pts2 = (rect.topleft, rect.topright, rect.bottomright)
+                pygame.draw.polygon(img, clr1, pts1)
+                pygame.draw.polygon(img, clr2, pts2)
+                img.set_colorkey((0,0,0))
+                
+                backw[(options[i], options[j])] = img
+                
+                img = pygame.transform.flip(img, True, False)
+                forw[(options[j], options[i])] = img
+                
+                img = pygame.transform.flip(img, False, True)
+                backw[(options[j], options[i])] = img
+                
+                img = pygame.transform.flip(img, True, False)
+                forw[(options[i], options[j])] = img
+                
+                j += 1
+            i += 1
+
+        # interfaceSurfaces: contains the surfaces used to blit the background
+        # colours for InterfaceMapBlocks. Each tuple key contains the two zone owners
+        # (in order) for which the corresponding surface is stored. Note that '\x00'
+        # indicates a neutral zone, whereas None indicates that there is no zone.
+        self.app.__interfaceSurfaces = {
+            ForwardInterfaceMapBlock: forw,
+            BackwardInterfaceMapBlock: backw
+        }
     
 class ForwardInterfaceMapBlock(InterfaceMapBlock):
     '''Represents a map block which contains the interface between two
@@ -813,55 +933,3 @@ _cornerInfo = [(GroundObstacle, ( 0, -1), GroundObstacle,     ( 1,  0)),
                (GroundObstacle, (-1, -1), GroundObstacle,     ( 1,  0))
                ]
 
-options = ('A', 'B', '\x00', None)
-colours = {'A' : team1bg,
-           'B' : team2bg,
-           '\x00' : neutral_bg,
-           None : (0,0,0)}
-forw = {}
-backw = {}
-i = 0
-while i < len(options):
-    # Skip ('A', 'B') by always starting j at least at '\x00'
-    j = max(i + 1, 2)
-    clr1 = colours[options[i]]
-    while j < len(options):
-        clr2 = colours[options[j]]
-        
-        img = pygame.surface.Surface((MapLayout.zoneInterfaceWidth, \
-                                      MapLayout.halfZoneHeight))
-        rect = img.get_rect()
-        pts1 = (rect.topleft, rect.bottomright, rect.bottomleft)
-        pts2 = (rect.topleft, rect.topright, rect.bottomright)
-        pygame.draw.polygon(img, clr1, pts1)
-        pygame.draw.polygon(img, clr2, pts2)
-        img.set_colorkey((0,0,0))
-        
-        backw[(options[i], options[j])] = img
-        
-        img = pygame.transform.flip(img, True, False)
-        forw[(options[j], options[i])] = img
-        
-        img = pygame.transform.flip(img, False, True)
-        backw[(options[j], options[i])] = img
-        
-        img = pygame.transform.flip(img, True, False)
-        forw[(options[i], options[j])] = img
-        
-        j += 1
-    i += 1
-
-
-
-# interfaceSurfaces: contains the surfaces used to blit the background
-# colours for InterfaceMapBlocks. Each tuple key contains the two zone owners
-# (in order) for which the corresponding surface is stored. Note that '\x00'
-# indicates a neutral zone, whereas None indicates that there is no zone.
-interfaceSurfaces = {ForwardInterfaceMapBlock: forw,
-                     BackwardInterfaceMapBlock: backw
-                    }
-
-del options
-del forw
-del backw
-del img
\ No newline at end of file
