@@ -143,6 +143,8 @@ class DiscoveryHandler(object):
         # lastPolled is the last server which was polled.
         self.lastPolled = None
 
+        self.recentPollers = RecentPollers()
+
         # servers is a list of (ipAddress, port) combinations which have are
         #   Internet addresses and have not been discovered to be offline or
         #   unreachable.
@@ -263,9 +265,8 @@ class DiscoveryHandler(object):
         '''
         if kind == self.dataKind or self.dataKind == '':
             # Store the data.
-            data = self.localData.setdefault(kind, {})
             address = ('127.0.0.1', self.netman.getTCPPort())
-            data[address, key] = DataValue(value)
+            self._doDataSet(kind, address, key, DataValue(value))
         assert self._dataValid()
 
         # Send the info on.
@@ -279,10 +280,8 @@ class DiscoveryHandler(object):
         '''
         if kind == self.dataKind or self.dataKind == '':
             # Store the data.
-            data = self.localData.setdefault(kind, {})
             address = ('127.0.0.1', self.netman.getTCPPort())
-            if (address, key) in data:
-                del data[address, key]
+            self._doDataRemove(kind, address, key, mustExist=False)
         assert self._dataValid()
 
         # Send the info on.
@@ -301,8 +300,10 @@ class DiscoveryHandler(object):
         servers, it will poll one instead.
         '''
         if len(self.peers) == 0:
-            # Not connected as peer to Internet
-            return self.pollServer(kind)
+            # Not connected as peer to Internet.
+            # Poll the Internet, but return current list.
+            self.pollServer(kind)
+            #return self.pollServer(kind)
 
         # Connected as peer, so just return the current list.
         result = self.getDataNow(kind)
@@ -325,8 +326,8 @@ class DiscoveryHandler(object):
             for kind, data in dta.iteritems():
                 for (svr, key) in data.keys():
                     if svr == server:
-                        del data[(svr, key)]
-            
+                        self._doDataRemove(kind, svr, key)
+
         if lanIP(server[0]):
             removeFrom(self.lanServers)
             removeFrom(self.lanPeers)
@@ -388,6 +389,24 @@ class DiscoveryHandler(object):
     # Other methods
     ################
 
+    def getDataDict(self, server):
+        if server == '127.0.0.1':
+            return self.localData
+        if lanIP(server):
+            return self.lanData
+        return self.data
+
+    def _doDataRemove(self, kind, server, key, mustExist=True):
+        data = self.getDataDict(server[0]).setdefault(kind, {})
+        if mustExist or (server, key) in data:
+            del data[server, key]
+            self.recentPollers.dataRemoved(kind, server, key)
+
+    def _doDataSet(self, kind, server, key, value):
+        assert isinstance(value, DataValue)
+        self.getDataDict(server[0]).setdefault(kind, {})[(server, key)] = value
+        self.recentPollers.dataAdded(kind, server, key, value)
+
     def addServer(self, address):
         host, port = address
         if lanIP(host):
@@ -449,10 +468,13 @@ class DiscoveryHandler(object):
             log('Lost Internet peer: %s' % (ipAddr,))
 
     def delServer(self, server):
-        if lanIP(server[0]):
-            self.lanServers.remove(server)
-        else:
-            self.servers.remove(server)
+        try:
+            if lanIP(server[0]):
+                self.lanServers.remove(server)
+            else:
+                self.servers.remove(server)
+        except ValueError:
+            pass
 
     def isPeer(self, ipAddr):
         for peer in self.peers:
@@ -484,7 +506,7 @@ class DiscoveryHandler(object):
             return False
 
         # Update the database.
-        data[stdkey] = DataValue(msg.value)
+        self._doDataSet(msg.kind, msg.address, msg.key, DataValue(msg.value))
         assert self._dataValid()
         return True
 
@@ -506,7 +528,7 @@ class DiscoveryHandler(object):
             return False
 
         # Update the database.
-        del data[stdkey]
+        self._doDataRemove(msg.kind, msg.address, msg.key)
         return True
 
     def resetData(self, kind, lan):
@@ -519,6 +541,7 @@ class DiscoveryHandler(object):
                 self.lanData[kind] = {}
             self.data[kind] = {}
         assert self._dataValid()
+        self.recentPollers.clear()
 
     def sendToPeers(self, msg, lanOnly=False, ignoreIPs=()):
         def sendIt(peers):
@@ -614,7 +637,7 @@ class DiscoveryHandler(object):
                 result.append(peers.pop(i))
 
         return result
-    
+
     def rememberServers(self, msg, lan):
         # Remember the suggested servers.
         for addr, port in msg.server1, msg.server2, msg.server3:
@@ -622,6 +645,66 @@ class DiscoveryHandler(object):
                 continue
             self.addServer((addr, port))
 
+class RecentPollers(object):
+    '''
+    Keeps track of clients who have recently polled this server.
+    '''
+    def __init__(self):
+        self._pollData = {}
+        self._pollTimes = {}
+
+    def clear(self):
+        self._pollData = {}
+        self._pollTimes = {}
+
+    def isPoller(self, ipAddr, port):
+        return (ipAddr, port) in self._pollTimes
+
+    def getPollData(self, ipAddr, port):
+        if not self.isPoller(ipAddr, port):
+            return None
+        result = self._pollData[ipAddr, port]
+        self.newPoller(ipAddr, port)
+        return result
+
+    def newPoller(self, ipAddr, port):
+        self._pollData[ipAddr, port] = []
+        self._pollTimes[ipAddr, port] = timeNow()
+        self.purge()
+
+    def purge(self):
+        limit = timeNow() - 30
+        for server, t in self._pollTimes.items():
+            if t < limit:
+                del self._pollTimes[server]
+                del self._pollData[server]
+
+    def getPollerCount(self):
+        return len(self._pollData)
+
+    def shouldAcceptPoll(self, ipAddr, port):
+        if self.isPoller(ipAddr, port):
+            return True
+        excessPollers = self.getPollerCount() - 2
+        if excessPollers <= 0:
+            return True
+        prob = 0.8 ** excessPollers
+        return random.random() < prob
+
+    def dataRemoved(self, kind, server, key):
+        lanData = lanIP(server[0]) and server[0] != '127.0.0.1'
+        msg = DeleteDataMsg(server, kind, key)
+        for (host, port), data in self._pollData.iteritems():
+            if lanIP(host) or not lanData:
+                data.append(msg)
+
+    def dataAdded(self, kind, server, key, value):
+        lanData = lanIP(server[0]) and server[0] != '127.0.0.1'
+        msg = UpdateDataMsg(server, kind, key, value.value)
+        for (host, port), data in self._pollData.iteritems():
+            if lanIP(host) or not lanData:
+                data.append(msg)
+
 class UDPMulticaster(object):
     def __init__(self, onAnnounce):
         # So that we can announce even if we can't listen.
@@ -638,7 +721,7 @@ class UDPMulticaster(object):
         except CannotListenError:
             # Cannot listen to the multicast, possibly because another
             # instance is running on this computer.
-            
+
             # Try listening again in 5 seconds.
             reactor.callLater(5, self.tryListening)
 
@@ -664,7 +747,7 @@ class UDPMulticastListener(DatagramProtocol):
         if datagram.startswith('GameDiscoveryProtocol1:Hello:'):
             tcpPort = struct.unpack('!I', datagram[29:33])[0]
             self.onAnnounce((address[0], tcpPort))
-            
+
 class UDPMulticastAnnouncer(DatagramProtocol):
     def announce(self, tcpPort):
         # Tell the network that we exist.
@@ -901,7 +984,7 @@ class PeerAttempt(object):
             # Denied as a peer of this host.
             if self._timer is not None:
                 self._timer.cancel()
-            
+
             # Remember the suggested peers.
             self.discoverer.rememberServers(msg, lanIP(self.server[0]))
 
@@ -923,13 +1006,14 @@ class PeerAttempt(object):
         self.failure()
 
         # NEXT: connectionLost
-    
+
 class IncomingConnection(object):
     def __init__(self, discoverer, connId, ipAddr, port):
         self.discoverer = discoverer
         self.netman = discoverer.netman
         self.connId = connId
         self.ipAddr = ipAddr
+        self.port = port
 
         # Set a time-out on this connection.
         self.timeout = reactor.callLater(10, self.timedOut)
@@ -1025,8 +1109,24 @@ class IncomingConnection(object):
         '''
         log('Received poll from: %s' % (self.ipAddr,))
 
-        # For now, just clear the list and send the entire list.
-        # Ideally you would only send changes for recent pollers.
+        recentPollers = self.discoverer.recentPollers
+
+        if not recentPollers.shouldAcceptPoll(self.ipAddr, self.port):
+            log('Denying poll')
+            self.netman.sendTCP(self.connId, RefuseQueryMsg(
+                    *self.discoverer.get3Peers(lanIP(self.ipAddr))))
+            return
+
+        pollData = recentPollers.getPollData(self.ipAddr, self.port)
+        if pollData is not None:
+            # We have stored poll data for this client.
+            for msg in pollData:
+                self.netman.sendTCP(self.connId, msg)
+            self.netman.sendTCP(self.connId, QueryCompleteMsg())
+            return
+
+        # New poller.  Just clear the list and send the entire list.
+        recentPollers.newPoller(self.ipAddr, self.port)
         self.netman.sendTCP(self.connId, ResetDataMsg())
 
         def sendSomeData(data, kind):
@@ -1184,7 +1284,7 @@ class ExpiringDataStore(object):
     passed around is not data you're interested in, you don't pass on multiple
     messages about the same data event.
     '''
-    
+
     def __init__(self):
         self.data = {}  # (kind, host, key) -> (value, callLater)
 
@@ -1193,14 +1293,14 @@ class ExpiringDataStore(object):
         result = True
         if (kind, host, key) in self.data:
             oldValue, oldCallLater = self.data[kind, host, key]
-            
+
             # Check if this is news.
             if oldValue == value:
                 result = False
 
             # Cancel the old callLater.
             oldCallLater.cancel()
-            
+
         # Create new entry.
         callLater = reactor.callLater(30, self._expired, (kind, host, key))
         self.data[kind, host, key] = (value, callLater)
@@ -1411,11 +1511,11 @@ def main(enableLogging=True):
         print 'Usage:'
         print '  %s [tcp_port [udp_port]]' % (sys.argv[0],)
         return
-    
+
     # Start the discoverer.
     print 'Starting game discovery service...'
     netman = NetworkManager(tPort, uPort)
-    handler = DiscoveryHandler(netman, 'discovery-server-0.0.0')
+    handler = DiscoveryHandler(netman, 'discovery-server-0.0.1')
 
     reactor.run()
 
