@@ -15,6 +15,7 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 # 02110-1301, USA.
 
+import copy
 import struct
 import random
 import marshal
@@ -120,6 +121,9 @@ class ServerCommandInterface(object):
         self.server = server
 
     def setGameMode(self, gameMode):
+        # TODO: when we swap over to using the same universe
+        # on both client and server side, this will need to
+        # send a message, not call a function.
         return self.server.setGameMode(gameMode)
 
     def kickPlayer(self, playerId):
@@ -808,9 +812,9 @@ class ServerNetHandler(object):
             self.streaks[ipAddr] = {}
             self.statTimers[ipAddr] = {}
         if nick not in self.stats[ipAddr]:         
-            self.stats[ipAddr][nick] = self.statList.copy()
-            self.streaks[ipAddr][nick] = self.streakList.copy()
-            self.statTimers[ipAddr][nick] = self.statTimerList.copy()
+            self.stats[ipAddr][nick] = copy.deepcopy(self.statList)
+            self.streaks[ipAddr][nick] = copy.deepcopy(self.streakList)
+            self.statTimers[ipAddr][nick] = copy.deepcopy(self.statTimerList)
 
     def addStat(self, statName, ipAddr, nick, increase = 1, enemy = None):
         if self.statList[statName] is not None:
@@ -991,16 +995,34 @@ class ServerNetHandler(object):
     def receiveJoinRequestMsg(self, connId, msg):
         # FIXME: do it per team
         # Check if we can add another player.
-        team = self.world.teamWithId[msg.teamId]
+
+        # Auto-join
+        if msg.auto == 1:
+            teamA = self.world.teamWithId['A']
+            teamB = self.world.teamWithId['B']
+            countA = len(teamA.connIds)
+            countB = len(teamB.connIds)
+            
+            if countA > countB:
+                team = teamB
+            elif countB > countA:
+                team = teamA
+            else:
+                team = random.choice([teamA, teamB])
+
+            msg.teamId = team.id
+
+        else:
+            team = self.world.teamWithId[msg.teamId]
      
         # Too many players, or game ended. Send response.
         if len(team.connIds) >= self.settings['maxPlayers']:
             self.netman.sendTCP(connId, 
-                    CannotCreatePlayerMsg('F', msg.teamId, nick=msg.nick))
+                CannotCreatePlayerMsg('F', msg.teamId, auto=msg.auto, nick=msg.nick))
             return
         elif self.world.gameState == GameState.Ended:
             self.netman.sendTCP(connId,
-                    CannotCreatePlayerMsg('O', msg.teamId, nick=msg.nick))
+                CannotCreatePlayerMsg('O', msg.teamId, auto=msg.auto, nick=msg.nick))
             return
 
         ipAddr = self.connectedClients[connId]
@@ -1014,10 +1036,22 @@ class ServerNetHandler(object):
                 timeLeft = REJOIN_DELAY_TIME - (timeNow() - leftTime)
                 if timeLeft > 0:
                     self.netman.sendTCP(connId,
-                            CannotCreatePlayerMsg('W', msg.teamId, timeLeft, msg.nick))
-                    print 'Player blocked from joining; needs to wait ' + str(timeLeft) + ' seconds'
+                            CannotCreatePlayerMsg('W', msg.teamId, timeLeft, msg.auto, msg.nick))
+                    #print 'Player blocked from joining; needs to wait ' + str(timeLeft) + ' seconds'
                     return
-            
+
+            # Stop the player from joining if their selected nickname is already in use
+            # (Testmode servers not included)
+            nameTaken = False
+            for player in self.world.playerWithId.values():
+                if player.nick.lower() == msg.nick.lower():
+                    nameTaken = True
+                    break
+
+            if nameTaken:
+                self.netman.sendTCP(connId, CannotCreatePlayerMsg('N', msg.teamId, auto=msg.auto, nick=msg.nick))
+                return
+        
         # Create a player id.
         playerId = self.newPlayerId(connId)
         
@@ -1025,10 +1059,10 @@ class ServerNetHandler(object):
         zone = self.world.getRandomZone(team)
 
         # First tell everyone that a new player has been added.
-        self.broadcastTcp(AddPlayerMsg(playerId, msg.teamId, zone.id, msg.nick))
+        self.broadcastTcp(AddPlayerMsg(playerId, msg.teamId, zone.id, 1, msg.nick))
         
         # Now tell this client that it owns the player.
-        self.netman.sendTCP(connId, GivePlayerMsg(playerId))
+        self.netman.sendTCP(connId, GivePlayerMsg(playerId, msg.auto))
         
         team.addAddress(playerId, connId)
 
@@ -1162,11 +1196,19 @@ class ServerNetHandler(object):
                 self.addStat('playerDeaths', victimIP, player.nick, enemy = killer.nick)
                              
                 self.broadcastUdp(KilledMsg(msg.targetId, msg.killerId,
-                        msg.shotId, msg.xPos, msg.yPos, killer.stars))
+                        msg.shotId, msg.xPos, msg.yPos))
+
+                if player.team.currentTransaction is not None:
+                    if (player.team.currentTransaction.purchasingPlayer ==
+                            player):
+                        self.sendTransactionUpdate(player.team, 'a',
+                                reason='death')
+
+                    if (player.team.currentTransaction.requiredStars <
+                            self.world.getTeamStars(player.team)):
+                        self.sendTransactionUpdate(player.team, 'a',
+                                reason='stars')
 
-                if player.team.currentTransaction and \
-                   player.team.currentTransaction.purchasingPlayer == player:
-                    self.sendTransactionUpdate(player.team, 'a')
                 killer.destroyShot(msg.shotId)
             elif player.dead:
                 self.validatePlayers(connId)
@@ -1263,8 +1305,6 @@ class ServerNetHandler(object):
 
         # Check zone ownership.
         zone = self.world.zoneWithId[msg.zoneId]
-        # TODO: remove the team attribute from the network message, since it's
-        # derivable from player anyway
         team = self.world.teamWithId[msg.teamId]
 
         # Check player status.
@@ -1420,10 +1460,10 @@ class ServerNetHandler(object):
         
         team = self.world.teamWithId[msg.teamId]
         transaction = team.currentTransaction
-        transaction.abandon()
+        transaction.abandon("request")
             
     def sendTransactionUpdate(self, team, update, player = None,
-                              stars = None, connId = None):
+                              stars = None, connId = None, reason = None):
         '''Should be called with update being:
         'a': abandon transaction
         'c': complete transaction
@@ -1436,7 +1476,12 @@ class ServerNetHandler(object):
             msg = AddedStarsMsg(team.id, player.id, stars, transaction.totalStars)
             
         elif update == 'a':
-            msg = AbandonTransactionMsg(team.id)
+            if reason is None:
+                if transaction.abandonReason is None:
+                    reason = "unknown"
+                else:
+                    reason = transaction.abandonReason
+            msg = AbandonTransactionMsg(team.id, reason)
             
             # Send the message to everyone, not just the team.
             team = None
@@ -1447,10 +1492,9 @@ class ServerNetHandler(object):
             msg = TransactionCompleteMsg(team.id,
                     transaction.purchasingPlayer.id,
                     transaction.upgrade.upgradeType)
-                             
-            # Update clients on new number of stars each player has.
-            # TODO: This should be implemented as client-side calcluations.
-            self.validateStars()
+
+            # TODO: need to remove this, yet still validate stars for enemy.
+            #self.validateStars() 
 
             # Record the stats.
             for player in transaction.contributions.iterkeys():
@@ -1502,7 +1546,6 @@ class ServerNetHandler(object):
             self.validateUpgrades(connId)
 
         player.upgrade.serverDelete()
-        self.broadcastUdp(msg)
 
     def sendDeleteUpgrade(self, playerId):
         self.broadcastUdp(DeleteUpgradeMsg(playerId))
@@ -1634,32 +1677,10 @@ class ServerNetHandler(object):
             print e, 'in message from %s' % (player.nick)
             return
 
-        if text.startswith('@@@') and self.settings['testMode']:
-            print 'Got info request via chat from %s' % player.nick
-            for teamID in ['A', 'B']:
-                team = self.world.teamWithId[teamID]
-                transaction = team.currentTransaction
-                transText = {
-                    TransactionState.Open : 'Open',
-                    TransactionState.Abandoned : 'Abandoned',
-                    TransactionState.Completed : 'Completed'
-                    }
-                messages = []
-                if transaction is not None:
-                    messages.append('Team%s has a transaction:' % teamID)
-                    messages.append('- Current state: %s' % transText[transaction.state])
-                    messages.append('- %s being purchased by %s' % (repr(transaction.upgrade), transaction.purchasingPlayer.nick))
-                    messages.append('- %s/%s stars' % (transaction.totalStars, transaction.requiredStars))
-                    messages.append('- Contributions: %s' %   transaction.contributions)
-                else:
-                    messages.append('Team%s has no transaction.' % teamID)
-
-                print "\n".join(messages)
-                
-                for messageStr in messages:
-                    sendId = self.playerConn[msg.senderId]
-                    self.netman.send(sendId, ChatFromServerMsg(messageStr.encode()), sendAddress)
-                    
+        # Not currently used but could be useful in the future for debugging
+        if text.startswith('@@@'):
+            print 'Got info request via chat from %s' % player.nick    
+            pass
         elif msg.kind == 't':
             team = self.world.teamWithId[msg.targetId]
             self.broadcastUdp(msg, team, connId)
@@ -1729,13 +1750,10 @@ class ServerNetHandler(object):
         if player.team.teamReady(player):
             # Rebroadcast the message.
             self.broadcastTcp(msg)
-        else:
-            # Player is not captain etc.
-            self.validateGameStatus(connId)
 
-        # Check if the both teams are ready.
-        if self.world.checkStart():
-            self.startSoon()
+            # Check if the both teams are ready.
+            if self.world.checkStart():
+                self.startSoon()
 
     def startSoon(self, reason = None, delay = 7):
         self.broadcastTcp(StartingSoonMsg(delay))
