@@ -28,7 +28,7 @@ from trosnoth.src.trosnothgui.ingame.udpnotify import UDPNotificationBar
 from trosnoth.src.trosnothgui.ingame import mainMenu
 from trosnoth.src.trosnothgui.ingame.transactionGUI import TransactionGUI
 from trosnoth.src.trosnothgui.ingame.gauges import TurretGauge, RespawnGauge, UpgradeGauge
-from trosnoth.src.trosnothgui.ingame import colours
+from trosnoth.src.trosnothgui.pregame.settingsMenu import SettingsMenu
 
 from trosnoth.src.base import MenuError
 import trosnoth.src.model.transaction as transactionModule
@@ -36,6 +36,9 @@ from trosnoth.src.model.upgrades import Turret, Shield, MinimapDisruption, Phase
 from trosnoth.src.model.team import Team
 from trosnoth.src.model.player import Player
 
+from trosnoth.src.network.transactions import *
+from trosnoth.src.messages.game import *
+
 class DetailsInterface(framework.CompoundElement):
     '''Interface containing all the overlays onto the screen:
     chat messages, player lists, gauges, stars, transaction requests, etc.'''
@@ -67,12 +70,17 @@ class DetailsInterface(framework.CompoundElement):
         self.upgradeGauge = None
         self.starGroup = StarGroup(self.app)
         self.udpNotifier = UDPNotificationBar(self.app)
+        self.settingsMenu = SettingsMenu(app, onClose=self.hideSettings, showThemes=False)
 
         menuloc = Location(FullScreenAttachedPoint((0,0), 'bottomleft'), 'bottomleft')
         self.menuManager = mainMenu.MainMenu(self.app, menuloc, self, self.gameInterface.keyMapping)
         self.elements = [self.currentMessages, self.currentChats,
                          self.starGroup, self.udpNotifier]
 
+    @property
+    def controller(self):
+        return self.gameInterface.controller
+
     def gameOver(self, winningTeam):
         self.gameInterface.gameOver(winningTeam)
 
@@ -81,6 +89,17 @@ class DetailsInterface(framework.CompoundElement):
         if self.menuManager not in self.elements:
             self.elements.append(self.menuManager)
 
+    def showSettings(self):
+        if self.settingsMenu not in self.elements:
+            self.elements.append(self.settingsMenu)
+            self.gameInterface.allowMouseGrab(False)
+
+    def hideSettings(self):
+        if self.settingsMenu in self.elements:
+            self.elements.remove(self.settingsMenu)
+            self.gameInterface.allowMouseGrab(True)
+            self.gameInterface.updateKeyMapping()
+
     def _getUpgrade(self, stars, upgrade):
         try:
             startingStars = int(stars)
@@ -90,7 +109,7 @@ class DetailsInterface(framework.CompoundElement):
             if not self.player.hasThisManyStars(startingStars):
                 raise MenuError, "You do not have that many stars"
             else:
-                self.gameInterface.netClient.startTransaction(upgrade, self.player, stars)
+                self.controller.send(StartingTransactionMsg(self.player.team.id, self.player.id, stars, upgrade.upgradeType))
 
     def _requestUpgrade(self, upgradeType):
         result, reason = self.player.canRequestUpgrade(upgradeType)
@@ -102,7 +121,7 @@ class DetailsInterface(framework.CompoundElement):
     def _addStars(self, numStars):
         result, reason = self.player.canAddStarsNow(numStars)
         if result:
-            self.gameInterface.netClient.addStars(self.player.team, self.player, numStars)
+            self.controller.send(AddingStarsMsg(self.player.team.id, self.player.id, numStars))
         else:
             raise MenuError, reason
         
@@ -122,6 +141,8 @@ class DetailsInterface(framework.CompoundElement):
         elif action == 'more actions':
             if self.menuManager is not None:
                 self.menuManager.showMoreMenu()
+        elif action == 'settings':
+            self.showSettings()
         elif action == 'leave':
             # Disconnect from the server.
             self.gameInterface.disconnect()
@@ -142,7 +163,8 @@ class DetailsInterface(framework.CompoundElement):
             if result:
                 self.player.requestRespawn()
             else:
-                raise MenuError, reason
+                self.newMessage(reason,
+                        self.app.theme.colours.errorMessageColour)
         elif action == 'turret':
             self._requestUpgrade(Turret)
             self.menuManager.manager.reset()
@@ -166,14 +188,16 @@ class DetailsInterface(framework.CompoundElement):
             if result:
                 self.player.requestUse()
             else:
-                raise MenuError, reason
+                self.newMessage(reason,
+                        self.app.theme.colours.errorMessageColour)
             self.menuManager.manager.reset()
         elif action == 'add stars':
             result, reason = self.player.canAddStars()
             if result:
                 self.getStars('How many stars to contribute?', self._addStars)
             else:
-                raise MenuError, reason
+                self.newMessage(reason,
+                        self.app.theme.colours.errorMessageColour)
             
             self.menuManager.manager.reset()
         elif action == 'abandon':
@@ -196,57 +220,68 @@ class DetailsInterface(framework.CompoundElement):
         '''Called when a transaction is started, so it can display this
         information to the user'''
         # There may be an existing transaction which is completed/abandoned, but still on screen
-        if self.transactionGUI:
+        if self.transactionGUI is not None:
             if self.transactionGUI.transaction.state not in (transactionModule.TransactionState.Abandoned, transactionModule.TransactionState.Completed):
                 # We must have had some bad information
                 print 'We had incorrect transaction information.'
-            self.elements.remove(self.transactionGUI)
-            del self.transactionGUI
-            self.transactionGUI = None
+            self._deleteTransGUI(self.transactionGUI)
 
         # If we're not focusing on a player, do not show the GUI.
         # Although perhaps if we were 'focusing' on a team, we could...
         # Count that as a TODO.
-        if not (self.player and transaction.purchasingTeam is self.player.team):
+        if self.player is None or transaction.purchasingTeam is not self.player.team:
             return
 
 
         self.transactionGUI = TransactionGUI(self.app, Location(FullScreenAttachedPoint(ScaledSize(200,0), 'topleft'), 'topleft'), transaction, self.player)
         self.elements.append(self.transactionGUI)
 
-        def deleteTransGUI(transactionGUI):
-            '''Removes the TransactionGUI object from the screen'''
-            # This check is necessary in case another one was created
-            # while one was still up
-            if transactionGUI is self.transactionGUI:
-                if transactionGUI in self.elements:
-                    self.elements.remove(transactionGUI)
-                else:
-                    print 'SOMEWHAT BAD: transactionGUI not in elements'
-                self.transactionGUI = None
+        self.newMessage('%s purchasing %s' % (
+                transaction.purchasingPlayer.nick, upgradeNames[
+                transaction.upgrade]),
+                self.app.theme.colours.transactionMessageColour)
 
-        def transactionChanged(sender, state):
-            assert transaction is sender
-            if state in (transactionModule.TransactionState.Abandoned, transactionModule.TransactionState.Completed):
-                self.newMessage("%s's Transaction %s" % (repr(transaction.purchasingTeam), transactionModule.TransactionStateText[state]), colours.transactionMessageColour)
-                reactor.callLater(3, deleteTransGUI, self.transactionGUI)
-
-        # Set up a message to display to the screen when someone adds star(s)
-        fAddStarsMessage = lambda sender, player, stars: self.newMessage('%s added %d star%s' % (player.nick, stars, ('s', '')[stars == 1]), colours.transactionMessageColour)
-
-        self.newMessage('%s purchasing %s' % (transaction.purchasingPlayer.nick, upgradeNames[transaction.upgrade]), colours.transactionMessageColour)
-                                                                         
-        transaction.onNumStarsChanged.addListener(fAddStarsMessage)
-        transaction.onStateChanged.addListener(transactionChanged)
+    # Set up a message to display to the screen when someone adds star(s)
+    def addStars(self, player, stars):
+        if player.team == self.player.team:
+            self.newMessage('%s added %d star%s' % (player.nick, stars,
+                    ('s', '')[stars == 1]),
+                    self.app.theme.colours.transactionMessageColour)
+        else:
+            print 'INEFFICIENCY: being sent enemy\'s star additions'
 
+    def _deleteTransGUI(self, transactionGUI):
+        '''Removes the TransactionGUI object from the screen'''
+        # This check is necessary in case another one was created
+        # while one was still up
+        if transactionGUI is self.transactionGUI:
+            if transactionGUI in self.elements:
+                self.elements.remove(transactionGUI)
+            else:
+                print 'SOMEWHAT BAD: transactionGUI not in elements'
+            self.transactionGUI = None
+        else:
+            print 'WARNING: Wrong transactionGUI being deleted'
+            
     def transactionChanged(self, transaction):
         if self.transactionGUI is None or self.transactionGUI.transaction != transaction:
             self.transactionStarted(transaction)
-        else:
-            # It should automatically detect any changes.
-            pass
 
-    def newMessage(self, text, colour = colours.grey):
+    def transactionAbandoned(self, transaction):
+        self.newMessage("%s's Transaction Abandoned" % (repr(transaction.purchasingTeam)),
+                self.app.theme.colours.transactionMessageColour)
+        if self.transactionGUI is not None and self.transactionGUI.transaction == transaction:
+            reactor.callLater(3, self._deleteTransGUI, self.transactionGUI)
+
+    def transactionComplete(self, transaction):
+        self.newMessage("%s's Transaction Completed" % (repr(transaction.purchasingTeam)),
+                self.app.theme.colours.transactionMessageColour)
+        if self.transactionGUI is not None and self.transactionGUI.transaction == transaction:
+            reactor.callLater(3, self._deleteTransGUI, self.transactionGUI)
+
+    def newMessage(self, text, colour=None):
+        if colour is None:
+            colour = self.app.theme.colours.grey
         self.currentMessages.newMessage(text, colour)
         self.allMessages.append((text))
 
@@ -277,7 +312,8 @@ class DetailsInterface(framework.CompoundElement):
             print "Don't want this text"
             return
         allText = nick + extra + text
-        self.currentChats.newMessage(allText, sender.team.chatColour)
+        self.currentChats.newMessage(allText,
+                self.app.theme.colours.chatColour(sender.team))
 
     def sentChat(self, text, team, private):
         if not private:
@@ -300,7 +336,8 @@ class DetailsInterface(framework.CompoundElement):
             print "Don't want this text"
             return
         allText = extra + text
-        self.currentChats.newMessage(allText, team.chatColour)
+        self.currentChats.newMessage(allText,
+                self.app.theme.colours.chatColour(team))
 
     def processEvent(self, event):
         '''Event processing works in the following way:
@@ -315,7 +352,7 @@ class DetailsInterface(framework.CompoundElement):
         try:
             return super(DetailsInterface, self).processEvent(event)
         except MenuError, err:
-            self.newMessage(err.value, colours.errorMessageColour)
+            self.newMessage(err.value, self.app.theme.colours.errorMessageColour)
             self.endInput()
             return None
 
@@ -360,7 +397,10 @@ class DetailsInterface(framework.CompoundElement):
         function is the function that should be called when the number of
         stars has been inputted'''
         self.endInput()
-        self.unobtrusiveGetter = NumberGetter(self.app, Location(FullScreenAttachedPoint((0,0), 'center'), 'center'), message, self.app.screenManager.fonts.unobtrusivePromptFont, colours.unobtrusivePromptColour, 3)
+        self.unobtrusiveGetter = NumberGetter(self.app, Location(
+                FullScreenAttachedPoint((0,0), 'center'), 'center'), message,
+                self.app.screenManager.fonts.unobtrusivePromptFont,
+                self.app.theme.colours.unobtrusivePromptColour, 3)
         self.elements.append(self.unobtrusiveGetter)
         def gotValue(num):
             if self.unobtrusiveGetter:
@@ -376,6 +416,11 @@ class DetailsInterface(framework.CompoundElement):
         the transaction they initiated'''
         if player.upgrade:
             addOn = 'Upgrade'
+            if player.upgrade.inUse and type(player.upgrade) == Grenade:
+                self.newMessage('Cannot abandon an active grenade!',
+                        self.app.theme.colours.errorMessageColour)
+                raise MenuError, "Cannot abandon an active grenade!"
+                
         elif player.team.currentTransaction and \
                 player.team.currentTransaction.purchasingPlayer is player:
             addOn = 'Transaction'
@@ -384,7 +429,10 @@ class DetailsInterface(framework.CompoundElement):
 
         message = 'Really abandon your ' + addOn + ' (Y/N)'
         self.endInput()
-        self.unobtrusiveGetter = YesNoGetter(self.app, Location(FullScreenAttachedPoint((0,0), 'center'), 'center'), message, self.app.screenManager.fonts.unobtrusivePromptFont, colours.unobtrusivePromptColour, 3)
+        self.unobtrusiveGetter = YesNoGetter(self.app, Location(
+                FullScreenAttachedPoint((0,0), 'center'), 'center'), message,
+                self.app.screenManager.fonts.unobtrusivePromptFont,
+                self.app.theme.colours.unobtrusivePromptColour, 3)
         self.elements.append(self.unobtrusiveGetter)
 
         def gotValue(abandon):
@@ -399,6 +447,10 @@ class DetailsInterface(framework.CompoundElement):
 
 
     def upgradeUsed(self, player):
+        message = '%s is using %s' % (player.nick, \
+                                      repr(player.upgrade))
+        self.newMessage(message)
+        
         if player is self.player:
             if self.upgradeGauge is not None:
                 # Remove the existing one.
@@ -407,7 +459,12 @@ class DetailsInterface(framework.CompoundElement):
             self.upgradeGauge = UpgradeGauge(self.app, Area(FullScreenAttachedPoint(ScaledSize(0,-20), 'midbottom'), ScaledSize(100,30), 'midbottom'), player)
             self.elements.append(self.upgradeGauge)
 
-    def upgradeDestroyed(self, player):
+
+    def upgradeDestroyed(self, player, upgrade):
+        message = "%s's %s is gone" % (player.nick, \
+                                       repr(upgrade))
+        self.newMessage(message)
+        
         if player is self.player and self.upgradeGauge is not None:
             self.elements.remove(self.upgradeGauge)
             self.upgradeGauge = None
@@ -436,11 +493,11 @@ class DetailsInterface(framework.CompoundElement):
 
     def becomeCaptain(self):
         if self.player:
-            self.gameInterface.netClient.setAsCaptain(self.player)
+            self.controller.send(SetCaptain(self.player))
 
     def teamReady(self):
         if self.player:
-            self.gameInterface.netClient.teamReady(self.player)
+            self.controller.send(TeamIsReady(self.player))
 
     def showPlayerDetails(self):
         self.gameInterface.gameViewer.toggleLeaderBoard()
