@@ -18,20 +18,23 @@
 '''leaderboard.py - defines the LeaderBoard class which deals with drawing the
 leader board to the screen.'''
 
+import pygame
 from twisted.internet import reactor
 import struct
-import random
 
 from trosnoth.data import getPath, sprites
 import trosnoth.src.gui.framework.framework as framework
-from trosnoth.src.gui.common import *
+from trosnoth.src.gui.common import (Location, ScaledScalar,
+        FullScreenAttachedPoint, SizedImage, ScaledSize)
 from trosnoth.src.gui.framework import table
 from trosnoth.src.gui.framework.elements import PictureElement
-from trosnoth.src.gui.fonts.font import ScaledFont
-from trosnoth.src.trosnothgui.ingame import colours
 from trosnoth.src.model.universe_base import GameState
 
+# How often the leaderboard will update (in seconds)
+UPDATE_DELAY = 1.0
+
 class LeaderBoard(framework.CompoundElement):
+    
     def __init__(self, app, world, gameViewer):
         super(LeaderBoard, self).__init__(app)
         self.world = world
@@ -51,7 +54,7 @@ class LeaderBoard(framework.CompoundElement):
         self.playerTable.setBorderColour((0, 0, 0))
 
         self.playerTable.style.backColour = None
-        self.playerTable.style.foreColour = colours.leaderboardNormal
+        self.playerTable.style.foreColour = app.theme.colours.leaderboardNormal
         self.playerTable.style.font = self.app.screenManager.fonts.leaderboardFont
         self.playerTable.style.padding = (4, 1)
         self.playerTable.style.hasShadow = True
@@ -67,17 +70,18 @@ class LeaderBoard(framework.CompoundElement):
         self.playerTable.getColumn(1).setWidth(ScaledScalar(25))    # Star column
         self.playerTable.getColumn(2).setWidth(ScaledScalar(70))    # Upgrade column
 
-        self.colourLookup = {"A": colours.leaderboardBlue, "B": colours.leaderboardRed}
+        self.colourLookup = {"A": app.theme.colours.leaderboardBlue,
+                "B": app.theme.colours.leaderboardRed}
 
         self._resetPlayers()
 
         self.elements = [self.playerTable]
-        self.update()
+        self.update(True)
 
     def _resetPlayers(self):
         self.players = {"A": [], "B": []}
 
-    def update(self):
+    def update(self, loop=False):
         self._resetPlayers()
 
         if self.world.gameState == GameState.InProgress:
@@ -105,13 +109,8 @@ class LeaderBoard(framework.CompoundElement):
         self._sort("B")
         self._updateTable()
 
-        xPos = 4
-        yPos = self.gameViewer.miniMap.sRect.bottom + self.gameViewer.zoneBarHeight + 5 + random.randint(0, 200)
-        position = Location(FullScreenAttachedPoint((xPos, yPos), 'topright'), 'topright')
-
-        #self.playerTable.pos = position
-        
-        self.callDef = reactor.callLater(1, self.update)
+        if loop:
+            self.callDef = reactor.callLater(UPDATE_DELAY, self.update, True)
 
     def _sort(self, team):
 
@@ -146,6 +145,8 @@ class LeaderBoard(framework.CompoundElement):
 
         # Upgrades that the enemy team have that you are allowed to see
         visibleUpgrades = ["Minimap Disruption", "Turret"]
+        # Upgrades that are currently in use that you are allowed to see
+        visibleInUse = visibleUpgrades + ["Shield", "Grenade"]
 
         for team, players in [(self.friendlyTeam, self.players[self.friendlyTeam.id]),
                               (self.enemyTeam, self.players[self.enemyTeam.id])]:
@@ -176,10 +177,12 @@ class LeaderBoard(framework.CompoundElement):
                 if self.world.gameState == GameState.PreGame:
                     if team.ready:
                         self.playerTable[2][count].setText("ready")
-                        self.playerTable[2][count].style.foreColour = colours.leaderboardNormal
+                        self.playerTable[2][count].style.foreColour = \
+                                self.app.theme.colours.leaderboardNormal
                     else:
                         self.playerTable[2][count].setText("not ready")
-                        self.playerTable[2][count].style.foreColour = colours.leaderboardDead
+                        self.playerTable[2][count].style.foreColour = \
+                                self.app.theme.colours.leaderboardDead
 
                 if self.world.gameState == GameState.Ended:
                     if self.winner == team:
@@ -213,8 +216,11 @@ class LeaderBoard(framework.CompoundElement):
                     if player["upgrade"] is None:
                         upgradeText = ""
                     else:
-                        # Only show enemy upgrades if they are in the whitelist
-                        if team is self.enemyTeam and player["upgrade"] not in visibleUpgrades:
+
+                        # Check if the upgrade is in the appropriate whitelist.
+                        # If there's an easier way of checking this, I haven't found it.                        
+                        if team is self.enemyTeam and ((player["upgradeInUse"] and player["upgrade"] not in visibleInUse) \
+                                                        or (not player["upgradeInUse"] and player["upgrade"] not in visibleUpgrades)):
                             upgradeText = ""
                         else:
                             # Use the short upgrade name if necessary
@@ -223,24 +229,36 @@ class LeaderBoard(framework.CompoundElement):
                             else:
                                 upgradeText = "+ " + player["upgrade"].lower()
                         
-                    self.playerTable[2][count].setText(upgradeText)                 
+                    self.playerTable[2][count].setText(upgradeText)
+                    if player["upgradeInUse"]:
+                        self.playerTable[2][count].style.foreColour = \
+                                self.app.theme.colours.leaderboardNormal
+                    else:
+                        self.playerTable[2][count].style.foreColour = \
+                                self.app.theme.colours.leaderboardDead
 
                 if player["dead"]:
-                    self.playerTable[0][count].style.foreColour = colours.leaderboardDead
-                    self.playerTable[1][count].style.foreColour = colours.leaderboardDead
+                    self.playerTable[0][count].style.foreColour = \
+                            self.app.theme.colours.leaderboardDead
+                    self.playerTable[1][count].style.foreColour = \
+                            self.app.theme.colours.leaderboardDead
                 else:
-                    self.playerTable[0][count].style.foreColour = self.colourLookup[team.id]
-                    self.playerTable[1][count].style.foreColour = colours.leaderboardNormal
+                    self.playerTable[0][count].style.foreColour = \
+                            self.colourLookup[team.id]
+                    self.playerTable[1][count].style.foreColour = \
+                            self.app.theme.colours.leaderboardNormal
 
             # Add the "star total" row
-            if len(players) > 1 and (team is self.friendlyTeam or self.spectator) \
-            and self.world.gameState != GameState.PreGame:
+            if (len(players) > 1 and (team is self.friendlyTeam or
+                    self.spectator) and self.world.gameState !=
+                    GameState.PreGame):
                 count += 1
                 self.playerTable.addRow()
                 self.playerTable.getRow(count).setHeight(ScaledScalar(self.rowHeight))
                 self.playerTable[0][count].setText("Total:")
-                self.playerTable[1][count].setText(str(team.teamStars))
-                if team.teamStars == 1:
+                teamStars = self.world.getTeamStars(team)
+                self.playerTable[1][count].setText(teamStars)
+                if teamStars == 1:
                     noun = "star"
                 else:
                     noun = "stars"
@@ -284,7 +302,9 @@ class LeaderBoard(framework.CompoundElement):
                 pointA = (colPoint[0], rowPoint[1] - bottomMargin)
                 pointB = (pointA[0] + self.playerTable.getColumn(1)._getWidth(), pointA[1])
                 pointC, pointD = shift(pointA, pointB)
-                pygame.draw.line(surface, colours.leaderboardNormal, pointA, pointB, 1)
+                pygame.draw.line(surface,
+                        self.app.theme.colours.leaderboardNormal, pointA,
+                        pointB, 1)
                 pygame.draw.line(surface, (0, 0, 0), pointC, pointD, 1)
 
 def shift(pointA, pointB, down = 1, right = 1):
