@@ -26,7 +26,6 @@ from trosnoth.src.utils.utils import new, timeNow
 from trosnoth.src.utils.logging import writeException
 from trosnoth.src.model import universe
 from trosnoth.src.trosnothgui.ingame.minimap import MiniMap
-from trosnoth.src.trosnothgui.ingame import colours
 from trosnoth.src.gui.framework import framework, elements
 from trosnoth.src.trosnothgui.ingame.leaderboard import LeaderBoard
 from trosnoth.src.trosnothgui.ingame.statusBar import ZoneProgressBar
@@ -108,9 +107,21 @@ class ViewManager(framework.CompoundElement):
             return self.target.pos
         return self.target
 
+    def appResized(self):
+        '''
+        Called when the app has been resized.
+        '''
+        self.sRect = pygame.Rect((0,0), self.app.screenManager.size)
+        # Create a surface for storing the background stuff.
+        self.backdrop = pygame.surface.Surface(self.sRect.size).convert()
+        self.spareCanvas = pygame.surface.Surface(self.sRect.size).convert()
+        # Now fill the backdrop with what we're looking at now.
+        self.drawBackdropRect(self.sRect)
+
     def draw(self, screen):
         '''Draws the current state of the universe at the current viewing
         location on the screen.  Does not call pygame.display.flip()'''
+
         # Update where we're looking at.
         deltaBackdrop = self.updateFocus()
         
@@ -251,13 +262,16 @@ class ViewManager(framework.CompoundElement):
             plPos = self.target.pos
             plAngle = self.target.angleFacing
 
-            radius = self.target.ghostThrust * self.lengthFromPlayer
-            #distanceToTarget
-            #if radius == None or radius > self.lengthFromPlayer:
-            #    radius = self.lengthFromPlayer
-                
-            targetPt = (plPos[0] + radius * sin(plAngle),
-                        plPos[1] - radius * cos(plAngle))
+            if self.app.displaySettings.centreOnPlayer:
+                targetPt = plPos
+            else:
+                radius = self.target.ghostThrust * self.lengthFromPlayer
+                #distanceToTarget
+                #if radius == None or radius > self.lengthFromPlayer:
+                #    radius = self.lengthFromPlayer
+                    
+                targetPt = (plPos[0] + radius * sin(plAngle),
+                            plPos[1] - radius * cos(plAngle))
 
             # If the player no longer exists, look wherever we want.
             if self.target not in self.universe.players:
@@ -335,16 +349,20 @@ class ViewManager(framework.CompoundElement):
         if sTarget == 0:
             return (0, 0)
         
-        # Calculate the maximum velocity that will result in deceleration to
-        # reach target. This is based on v**2 = u**2 + 2as
-        vDecel = (2. * self.acceleration * sTarget) ** 0.5
-        
-        # Actual velocity is limited by this and maximum velocity.
-        self.speed = min(self.maxSpeed, vDecel, \
-                         self.speed + self.acceleration * deltaT)
-        
-        # Distance travelled should never overshoot the target.
-        s = min(sTarget, self.speed * deltaT)
+        # If smooth panning is switched off, jump to location.
+        if not self.app.displaySettings.smoothPanning:
+            s = sTarget
+        else:
+            # Calculate the maximum velocity that will result in deceleration to
+            # reach target. This is based on v**2 = u**2 + 2as
+            vDecel = (2. * self.acceleration * sTarget) ** 0.5
+            
+            # Actual velocity is limited by this and maximum velocity.
+            self.speed = min(self.maxSpeed, vDecel, \
+                             self.speed + self.acceleration * deltaT)
+            
+            # Distance travelled should never overshoot the target.
+            s = min(sTarget, self.speed * deltaT)
         
         # How far does the backdrop need to move by?
         #  (This will be negative what the focus moves by.)
@@ -370,7 +388,7 @@ class PregameMessage(elements.TextElement):
             'Game will begin when both captains signal ready.',
             app.screenManager.fonts.bigMenuFont,
             pos,
-            colours.pregameMessageColour,
+            app.theme.colours.pregameMessageColour,
             shadow = True
         )
         self.setShadowColour((0,0,0))
@@ -387,58 +405,91 @@ class GameViewer(framework.CompoundElement):
         super(GameViewer, self).__init__(app)
         self.interface = gameInterface
         self.world = world
-        self.worldgui = UniverseGUI(self.world)
+        self.worldgui = UniverseGUI(app, self.world)
         self.app = app
 
         self.viewManager = ViewManager(self.app, self.worldgui)
-        miniMapSize = (min(self.app.screenManager.scaledSize[0] / 100 * 35, 300),
-                    min(self.app.screenManager.scaledSize[1] / 100 * 35, 200))
-        self.miniMap = MiniMap(self.app, (self.app.screenManager.size[0] - miniMapSize[0] - 5, 5),\
-                                       miniMapSize, 20, world, \
-                                       self.viewManager)
 
-        self.leaderboard = LeaderBoard(app, world, self)
-        self.zoneBar = ZoneProgressBar(app, world, self)
         self.timerBar = GameTimer(app, world)
         self.chatBox = ChatBox(self.app, self.world, self)
-
-        xPos = -10
-        yPos = self.miniMap.sRect.bottom + self.zoneBarHeight - 5
-        messagePos = Location(FullScreenAttachedPoint((xPos, yPos), 'topright'), 'topright')
-        self.pregameMessage = PregameMessage(self.app, messagePos)
-        if world.gameState == GameState.InProgress:
-            self.pregameMessage.setText('')
+        self.miniMap = None
+        self.pregameMessage = None
+        self.leaderboard = None
+        self.zoneBar = None
+        self.makeMiniMap()
 
         self.elements = [self.viewManager, self.pregameMessage]
 
         self.minimapDisruptor = None
+        self.teamsDisrupted = set()
 
         self.toggleInterface()
         self.toggleLeaderBoard()
+
+    def resizeIfNeeded(self):
+        '''
+        Checks whether the application has resized and adjusts accordingly.
+        '''
+        if self.viewManager.sRect.size == self.app.screenManager.size:
+            return
+        self.viewManager.appResized()
+        # Recreate the minimap.
+        showHUD = self.miniMap in self.elements
+        showLeader = self.leaderboard in self.elements
+        self.makeMiniMap()
+        if showHUD:
+            self.toggleInterface()
+        if showLeader:
+            self.toggleLeaderBoard()
+            
+
+    def makeMiniMap(self):
+        miniMapSize = (min(self.app.screenManager.scaledSize[0] / 100 * 35, 300),
+                    min(self.app.screenManager.scaledSize[1] / 100 * 35, 200))
+        self.miniMap = MiniMap(self.app, (self.app.screenManager.size[0] - miniMapSize[0] - 5, 5),
+                                       miniMapSize, 20, self.world,
+                                       self.viewManager)
+        xPos = -10
+        yPos = self.miniMap.sRect.bottom + self.zoneBarHeight - 5
+        messagePos = Location(FullScreenAttachedPoint((xPos, yPos), 'topright'), 'topright')
+        self.pregameMessage = PregameMessage(self.app, messagePos)
+        if self.world.gameState == GameState.InProgress:
+            self.pregameMessage.setText('')
+        self.zoneBar = ZoneProgressBar(self.app, self.world, self)
+        self.leaderboard = LeaderBoard(self.app, self.world, self)
+
+        self.elements = [self.viewManager, self.pregameMessage]
                 
     def setTarget(self, target):
         'Target should be a player, a point, or None.'
         self.viewManager.setTarget(target)
         if isinstance(target, PlayerSprite):
-            if self.minimapDisruptor:
-                if target.team != self.minimapDisruptor.team:
-                    self.miniMap.disrupted()
-                else:
-                    self.miniMap.endDisruption()
+            if target.team in self.teamsDisrupted:
+                self.miniMap.disrupted()
+            else:
+                self.miniMap.endDisruption()
         
     def tick(self, deltaT):
+        self.resizeIfNeeded()
         self.worldgui.tick(deltaT)
         super(GameViewer, self).tick(deltaT)
 
     def minimapDisruption(self, player):
-        self.minimapDisruptor = player
+        disruptedTeam = player.team.opposingTeam
+        self.teamsDisrupted.add(disruptedTeam)
         if isinstance(self.viewManager.target, PlayerSprite):
-            if player.team != self.viewManager.target.team:
+            if self.viewManager.target.team == disruptedTeam:
                 self.miniMap.disrupted()
+                self.zoneBar.disrupt = True
+                self.minimapDisruptor = player
 
-    def endMinimapDisruption(self):
-        self.minimapDisruptor = None
-        self.miniMap.endDisruption()
+    def endMinimapDisruption(self, player):
+        disruptedTeam = player.team.opposingTeam
+        self.teamsDisrupted.remove(disruptedTeam)
+        if self.viewManager.target.team == disruptedTeam:
+            self.minimapDisruptor = None
+            self.miniMap.endDisruption()
+            self.zoneBar.disrupt = False
 
     def toggleInterface(self):
         if self.miniMap in self.elements:
