@@ -36,10 +36,8 @@ from trosnoth.src.gui.framework.elements import PictureElement, TextElement, Tex
 from trosnoth.src.gui.framework.tab import Tab
 from trosnoth.src.gui.framework.checkbox import CheckBox
 from trosnoth.src.gui.framework.listbox import ListBox
-from trosnoth.src.gui.framework.dialogbox import DialogBox, DialogResult, DialogBoxAttachedPoint
+from trosnoth.src.gui.framework.dialogbox import DialogBox, DialogResult, DialogBoxAttachedPoint, OkBox
 from trosnoth.data import startupMenu
-from trosnoth.src.trosnothgui.pregame import colours
-from trosnoth.src.gui.fonts.font import ScaledFont
 
 from trosnoth.src.gui.common import *
 
@@ -123,6 +121,7 @@ class ServerDoesntExistMenu(framework.TabFriendlyCompoundElement):
         self.radio.onSelectionChanged.addListener(self.setSelectionNumber)   
 
 
+        colours = app.theme.colours
         self.text = [
                      TextElement(self.app, 'Game Name:', font,
                                  ScaledLocation(85, 220),
@@ -175,13 +174,21 @@ class ServerDoesntExistMenu(framework.TabFriendlyCompoundElement):
                                            style='circle')
         self.noLimitBox.onValueChanged.addListener(lambda sender: self.enableGameDuration(not sender.value))
 
+        from trosnoth.src.utils.jsonImport import json
+        if json == None:
+            self.warnForRecording = True
+        else:
+            self.warnForRecording = False
+        del json
+        
         self.recordReplayBox = CheckBox(self.app, ScaledLocation(600, 520),
-                                           text = 'Save Replay',
-                                           font = app.screenManager.fonts.smallNoteFont,
-                                           colour = colours.mainMenuColour,
-                                           initValue = True,
-                                           style='circle')
-
+                                        text = 'Save Replay',
+                                        font = app.screenManager.fonts.smallNoteFont,
+                                        colour = colours.mainMenuColour,
+                                        initValue = not self.warnForRecording,
+                                        style='circle')
+        
+        self.recordReplayBox.onValueChanged.addListener(self.recordReplayButtonClicked)
         self.invisibleGameBox = CheckBox(self.app, ScaledLocation(800, 520),
                                            text = 'Invisible',
                                            font = app.screenManager.fonts.smallNoteFont,
@@ -215,6 +222,15 @@ class ServerDoesntExistMenu(framework.TabFriendlyCompoundElement):
         self.lastSelection = -1
         self.setSelectionNumber(0)   
 
+    def recordReplayButtonClicked(self, sender):
+        if self.warnForRecording and sender.value:
+            box = OkBox(self.app, ScaledSize(350, 200), "Warning!", "json is not installed.\nYour replay will not be saved.")
+            colours = self.app.theme.colours
+            box.setColours(borderColour = colours.errorColour, titleColour = colours.white)
+            box.Show()
+            self.warnForRecording = False
+            sender.setValue(False)
+                         
     def setSelectionNumber(self, index):
         playerSizes = [10, 4, 6]
         timeLimits = [45, 10, 20]
@@ -253,10 +269,11 @@ class ServerDoesntExistMenu(framework.TabFriendlyCompoundElement):
     def enableGameDuration(self, val):
         self._enableGameDuration = val
         if val:
-            self.gameDurationInput.setBackColour(colours.white)
+            self.gameDurationInput.setBackColour(self.app.theme.colours.white)
             self.setFocus(self.gameDurationInput)
         else:
-            self.gameDurationInput.setBackColour(colours.disabled)
+            self.gameDurationInput.setBackColour(
+                    self.app.theme.colours.disabled)
             if self.gameDurationInput.hasFocus:
                 self.setFocus(self.maxPlayersInput)
         self.setTabOrder()
@@ -276,11 +293,12 @@ class ServerDoesntExistMenu(framework.TabFriendlyCompoundElement):
 
     def setBackColours(self):
         if self._enableGameDuration:
-            self.gameDurationInput.setBackColour(colours.white)
+            self.gameDurationInput.setBackColour(self.app.theme.colours.white)
         else:
-            self.gameDurationInput.setBackColour(colours.disabled)
-        self.maxPlayersInput.setBackColour(colours.white)
-        self.serverInput.setBackColour(colours.white)
+            self.gameDurationInput.setBackColour(
+                    self.app.theme.colours.disabled)
+        self.maxPlayersInput.setBackColour(self.app.theme.colours.white)
+        self.serverInput.setBackColour(self.app.theme.colours.white)
 
     def getValues(self):
         '''
@@ -299,6 +317,7 @@ class ServerDoesntExistMenu(framework.TabFriendlyCompoundElement):
 
         invalid = False
         
+        colours = self.app.theme.colours
         height = width = maxPlayers = duration = name = None
         if self.serverInput.value == '':
             if not invalid:
@@ -359,9 +378,12 @@ class ServerDoesntExistMenu(framework.TabFriendlyCompoundElement):
         super(ServerDoesntExistMenu, self).draw(surface)
         rect = self.tabContainer._getTabRect()
         verLineX = rect.left + (rect.width * 0.545)
-        pygame.draw.line(surface, colours.tabContainerColour, (verLineX, rect.top), (verLineX, rect.bottom), self.tabContainer._getBorderWidth())
+        colour = self.app.theme.colours.tabContainerColour
+        pygame.draw.line(surface, colour, (verLineX, rect.top), (verLineX,
+                rect.bottom), self.tabContainer._getBorderWidth())
         horLineY = rect.top + (rect.height * 0.72)
-        pygame.draw.line(surface, colours.tabContainerColour, (verLineX, horLineY), (rect.right, horLineY), self.tabContainer._getBorderWidth())
+        pygame.draw.line(surface, colour, (verLineX, horLineY), (rect.right,
+                horLineY), self.tabContainer._getBorderWidth())
 
 
 class ServerAlreadyExistsMenu(framework.TabFriendlyCompoundElement):
@@ -384,6 +406,7 @@ class ServerAlreadyExistsMenu(framework.TabFriendlyCompoundElement):
         self._lastStatusUpdate = timeNow()
 
         font = self.app.screenManager.fonts.bigMenuFont
+        colours = app.theme.colours
 
         # Static text
         self.staticText = [TextElement(self.app, 'Game Status:', font,
@@ -438,7 +461,7 @@ class ServerAlreadyExistsMenu(framework.TabFriendlyCompoundElement):
 
         # Text buttons
         self.shutdownButton = TextButton(self.app, ScaledLocation(410, 620, 'center'),
-                                         'SHUT DOWN SERVER',
+                                         'Shut Down Server',
                                          app.screenManager.fonts.hugeMenuFont,
                                          colours.redTeam, colours.white)
         self.shutdownButton.onClick.addListener(lambda sender: self.shutdownServer())
@@ -462,6 +485,7 @@ class ServerAlreadyExistsMenu(framework.TabFriendlyCompoundElement):
         self.updateStatus()
        
     def saveSettings(self):
+        colours = self.app.theme.colours
         cmd = self.getCmdInterface()
         if cmd is None:
             self.assistanceText.setText('Game is not running!')
@@ -535,17 +559,19 @@ class ServerAlreadyExistsMenu(framework.TabFriendlyCompoundElement):
             self.stopCountdown('Settings reset due to inactivity')
             self.populateInputs()
             self.delayedErase(self.assistanceText)
-        elif self.resetCountdown <= 5 or self.resetCountdown == 10 or self.resetCountdown == 20:
+        elif (self.resetCountdown <= 5 or self.resetCountdown == 10 or
+                self.resetCountdown == 20):
             if self.resetCountdown == 1:
                 _noun = "second"
             else:
                 _noun = "seconds"
-            self.assistanceText.setText('Settings will reset in %d %s' % (self.resetCountdown, _noun))
-            self.assistanceText.setColour(colours.startButton)
+            self.assistanceText.setText('Settings will reset in %d %s' %
+                    (self.resetCountdown, _noun))
+            self.assistanceText.setColour(self.app.theme.colours.startButton)
 
     def stopCountdown(self, message):
         self.assistanceText.setText(message)
-        self.assistanceText.setColour(colours.invalidDataColour)
+        self.assistanceText.setColour(self.app.theme.colours.invalidDataColour)
         # There's a chance that the timer might never have been started.
         try:
             self.resetTimer.stop()
@@ -573,6 +599,7 @@ class ServerAlreadyExistsMenu(framework.TabFriendlyCompoundElement):
         return None
 
     def updateStatus(self):
+        colours = self.app.theme.colours
         status = self.gameStatusText
         cmd = self.getCmdInterface()
         if cmd is None:
@@ -635,32 +662,49 @@ class ShutdownDialog(DialogBox):
 
     def __init__(self, app):
         super(ShutdownDialog, self).__init__(app, ScaledSize(450, 340), 'Server Shutdown')
+        colours = app.theme.colours
 
         # Warning message
-        text = [TextElement(app, 'Are you sure you want', app.screenManager.fonts.mediumMenuFont,
-                                Location(DialogBoxAttachedPoint(self, ScaledSize(0, 10), 'midtop'), 'midtop'), colours.invalidDataColour),
-                TextElement(app, 'to shutdown the server?', app.screenManager.fonts.mediumMenuFont,
-                                Location(DialogBoxAttachedPoint(self, ScaledSize(0, 55), 'midtop'), 'midtop'), colours.invalidDataColour),
-                TextElement(app, 'Doing so will immediately', app.screenManager.fonts.mediumMenuFont,
-                                Location(DialogBoxAttachedPoint(self, ScaledSize(0,130), 'midtop'), 'midtop'), colours.invalidDataColour),
-                TextElement(app, 'end the game and', app.screenManager.fonts.mediumMenuFont,
-                                Location(DialogBoxAttachedPoint(self, ScaledSize(0,175), 'midtop'), 'midtop'), colours.invalidDataColour),
-                TextElement(app, 'disconnect all users!', app.screenManager.fonts.mediumMenuFont,
-                                Location(DialogBoxAttachedPoint(self, ScaledSize(0,220), 'midtop'), 'midtop'), colours.invalidDataColour)]
+        text = [
+            TextElement(app, 'Are you sure you want',
+                    app.screenManager.fonts.mediumMenuFont,
+                    Location(DialogBoxAttachedPoint(self, ScaledSize(0, 10),
+                    'midtop'), 'midtop'), colours.invalidDataColour),
+            TextElement(app, 'to shutdown the server?',
+                    app.screenManager.fonts.mediumMenuFont,
+                    Location(DialogBoxAttachedPoint(self, ScaledSize(0, 55),
+                    'midtop'), 'midtop'), colours.invalidDataColour),
+            TextElement(app, 'Doing so will immediately',
+                    app.screenManager.fonts.mediumMenuFont,
+                    Location(DialogBoxAttachedPoint(self, ScaledSize(0,130),
+                    'midtop'), 'midtop'), colours.invalidDataColour),
+            TextElement(app, 'end the game and',
+                    app.screenManager.fonts.mediumMenuFont,
+                    Location(DialogBoxAttachedPoint(self, ScaledSize(0,175),
+                    'midtop'), 'midtop'), colours.invalidDataColour),
+            TextElement(app, 'disconnect all users!',
+                    app.screenManager.fonts.mediumMenuFont,
+                    Location(DialogBoxAttachedPoint(self, ScaledSize(0,220),
+                    'midtop'), 'midtop'), colours.invalidDataColour)]
         
-        font = ScaledFont('KLEPTOCR.TTF', 50)
+        font = app.screenManager.fonts.dialogButtonFont
 
         # OK Button
-        okButton = TextButton(app,  Location(DialogBoxAttachedPoint(self, ScaledSize(-100,280), 'midtop'), 'midtop'), 'Confirm', font, colours.dialogButtonColour, colours.radioMouseover)
+        okButton = TextButton(app,  Location(DialogBoxAttachedPoint(self,
+                ScaledSize(-100,280), 'midtop'), 'midtop'), 'Confirm', font,
+                colours.dialogButtonColour, colours.radioMouseover)
         okButton.onClick.addListener(lambda sender: self.ok())
 
         # Cancel Button
-        cancelButton = TextButton(app,  Location(DialogBoxAttachedPoint(self, ScaledSize(100,280), 'midtop'), 'midtop'), 'Cancel', font, colours.dialogButtonColour, colours.radioMouseover)
+        cancelButton = TextButton(app,  Location(DialogBoxAttachedPoint(self,
+                ScaledSize(100,280), 'midtop'), 'midtop'), 'Cancel', font,
+                colours.dialogButtonColour, colours.radioMouseover)
         cancelButton.onClick.addListener(lambda sender: self.cancel())
 
         # Add elements to screen
         self.elements = text + [okButton, cancelButton]
-        self.setColours(colours.redTeam, colours.dialogBoxTextColour, colours.dialogBoxTextColour)
+        self.setColours(colours.redTeam, colours.dialogBoxTextColour,
+                colours.dialogBoxTextColour)
 
     def ok(self):
         self.result = DialogResult.OK
