@@ -25,15 +25,18 @@ import pygame
 import pygame.locals as pgLocals
 from math import sin, atan
 from copy import copy
-import traceback
 
-from trosnoth.src.universe import Universe, ForwardInterfaceMapBlock, InterfaceMapBlock, \
-     BackwardInterfaceMapBlock, TopBodyMapBlock, BottomBodyMapBlock, BodyMapBlock, MapLayout
+from trosnoth.src.model.universe import Universe
+from trosnoth.src.model.mapblocks import ForwardInterfaceMapBlock, InterfaceMapBlock, \
+     BackwardInterfaceMapBlock, TopBodyMapBlock, BottomBodyMapBlock, BodyMapBlock
+from trosnoth.src.model.map import MapLayout
 from trosnoth.src.gui.framework import elements, framework
 from trosnoth.src.gui import app
 from trosnoth.src.utils.utils import new
+from trosnoth.src.utils.math import distance
+from trosnoth.src.utils.unrepr import unrepr
 
-from trosnoth.data import getPath, blocks
+from trosnoth.data import getPath, makeDirs, user
 
 from trosnoth.tools.map_editor import interface
 
@@ -83,6 +86,9 @@ scales = [1.5, 1.25, 1, 0.75, 0.5]
 
 infinity = new(1)
 
+blockDir = getPath(user, 'blocks')
+makeDirs(blockDir)
+
 from math import sqrt, fabs
 def lineMagnitude(a, b):
     return sqrt((b[0] - a[0]) ** 2 + (b[1] - a[1]) ** 2)
@@ -113,9 +119,6 @@ def pointToLine(a, b, c):
 def initMenu(mapBlockCreator):
     pass
 
-def distance(pt1, pt2):
-    return ((pt1[0] - pt2[0]) ** 2 + (pt1[1] - pt2[1]) ** 2) ** 0.5
-
 def gradient(startPt, endPt):
     try:
         grad = (float(startPt[1] - endPt[1])) / (startPt[0] - endPt[0])
@@ -256,7 +259,7 @@ class MapBlockCreator(app.Application):
     minLength = 10
     scaleIndex = 3
     def __init__(self):
-        super(MapBlockCreator, self).__init__((600,800), 0, "hello", interface.Interface, False)
+        super(MapBlockCreator, self).__init__((600,800), 0, "hello", interface.Interface)
         
         self.halfPlayerSize = (22, 22)
         self.undoStack = UndoStack()
@@ -1284,14 +1287,15 @@ class MapBlockCreator(app.Application):
                 allObstacles.append(tuple(currentSet))
                 currentSet = []
                 
-        finalString = 'blockType = ' + self.currentType.__name__
-        finalString += ',\nblocked = ' + repr(self.currentBlockage)
-        finalString += ",\nsymmetrical = %s," % (self.symmetrical)
-        finalString += "\ngraphic = '%s'," % (self.graphicName)
-        finalString += "\nobstacles = %s" % (repr(allObstacles))
+        output = {
+            'blockType': self.currentType.__name__,
+            'blocked': self.currentBlockage,
+            'symmetrical': self.symmetrical,
+            'obstacles': allObstacles,
+        }
         if allPlatforms != []:
-            finalString += ",\nplatforms = %s" % (repr(allPlatforms))
-        return finalString
+            output['platforms'] = allPlatforms
+        return repr(output)
 
     def save(self):
         if not (self.path and self.graphicName and self.blockName):
@@ -1301,9 +1305,9 @@ class MapBlockCreator(app.Application):
 
     def saveInternal(self):
         try:
-            graphicPath = getPath(blocks, self.graphicName)
+            graphicPath = os.path.join(blockDir, self.graphicName)
             self.interface.graphify(graphicPath)
-            blockPath = getPath(blocks, self.blockName)
+            blockPath = os.path.join(blockDir, self.blockName)
             file = open(blockPath,'w')
             outputStr = self.getMapBlockString()
             file.write(outputStr)
@@ -1333,7 +1337,7 @@ class MapBlockCreator(app.Application):
 
         self.graphicName = filename + imgExt
         self.blockName = filename + blockExt
-        self.path = getPath(blocks)
+        self.path = blockDir
         self.saveInternal()
 
     def load(self):
@@ -1343,7 +1347,7 @@ class MapBlockCreator(app.Application):
     
             filename = tkFileDialog.askopenfilename(
                         parent=tk,
-                        initialdir = getPath(blocks),
+                        initialdir = blockDir,
                         defaultextension=blockExt, \
                         filetypes=[('Block File', '*.block')])
             tk.destroy()
@@ -1359,14 +1363,10 @@ class MapBlockCreator(app.Application):
                 f.close()
             
             try:
-                code = compile('loaded("%s", %s)' % (filename, contents), \
-                               filename, 'exec')
-            except SyntaxError:
+                contents = unrepr(contents)
+            except:
                 raise IOError, 'invalid file format'
-            
-            namespace = blockFileNamespace.copy()
-            namespace.update({'loaded': self.loaded})
-            exec code in namespace
+            self.loaded(filename, contents)
         except IOError, e:
             print e
             self.closeCurrentFile()
@@ -1376,8 +1376,13 @@ class MapBlockCreator(app.Application):
 
         
 
-    def loaded(self, filename, blockType=TopBodyMapBlock, blocked=False, \
-                  graphic=None, obstacles=[], platforms=[], symmetrical=False):
+    def loaded(self, filename, contents):
+        blockType = blockFileNamespace[contents.get('blockType', TopBodyMapBlock)]
+        blocked = contents.get('blocked', False)
+        obstacles = contents.get('obstacles', [])
+        platforms = contents.get('platforms', [])
+        symmetrical = contents.get('symmetrical', False)
+
         try:
             self.closeCurrentFile()
             for connectSet in obstacles:
@@ -1407,10 +1412,7 @@ class MapBlockCreator(app.Application):
             self.changeCreateType(blockType, blocked)
             self.setMandatoryFromLoad()
             self.path, self.blockName = self.extractPartsOfFilename(filename)
-            if graphic:
-                self.graphicName = graphic
-            else:
-                self.graphicName = self.extractFilename(self.blockName) + imgExt
+            self.graphicName = self.extractFilename(self.blockName) + imgExt
 
         except ValueError:
             raise IOError, "Something didn't load correctly"
@@ -1574,7 +1576,7 @@ blockFileNamespace = {'ForwardInterfaceMapBlock': ForwardInterfaceMapBlock,
                       'BottomBodyMapBlock': BottomBodyMapBlock}
 
 
-imgExt = '.bmp'
+imgExt = '.png'
 blockExt = '.block'
 
 def run():
